package com.zh.project0512.controller;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.mapper.QywxDepartmentUserLinkMapper;
import com.zh.project0512.mapper.RankMapper;
import com.zh.project0512.model.Rank;
import com.zh.project0512.model.Task;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.model.vo.RankListVo;
import com.zh.project0512.service.ITaskService;
import com.zh.project0512.utils.PropertyUtils;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

@Tag(name = "定时任务")
@RestController
public class ScheduleController {
    @Autowired
    private ITaskService taskService;
    @Resource
    QywxDepartmentUserLinkMapper qywxDepartmentUserLinkMapper;
    @Resource
    RankMapper rankMapper;

    @Scheduled(cron = "0 0 0 * * ?")
    public void taskBegin() {
        UpdateWrapper<Task> ew = new UpdateWrapper<>();
        LocalDateTime now = LocalDateTime.now();
        ew.eq("status",0).eq("date_format(start, '%Y-%m-%d')", now.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")))
            .set("status",1).set("updateAt",now);
        taskService.update(ew);
    }
    @Scheduled(cron = "0 0 0 * * ?")
    public void taskEnd() {
        UpdateWrapper<Task> ew = new UpdateWrapper<>();
        LocalDateTime now = LocalDateTime.now();
        ew.eq("status",1).eq("date_format(end, '%Y-%m-%d')", now.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")))
            .set("status",2).set("updateAt",now);
        taskService.update(ew);
    }

//    @Scheduled(cron = "0 0/1 * * * ? ")
    @Scheduled(cron = "0 0 0 1 1/1 ?")
    public void taskRank() {
        //获取开始时间和结束时间
        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.add(Calendar.MONTH,-1);
        cal.set(Calendar.DAY_OF_MONTH,1);
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        Date timeStart = cal.getTime();
        cal.set(Calendar.DAY_OF_MONTH,cal.getActualMaximum(Calendar.DAY_OF_MONTH));
        Date timeEnd = cal.getTime();
        //获取员工排名进排名数据库
        QueryWrapper<RankListDTO> queryWrapper = new QueryWrapper<>();
        queryWrapper.groupBy("u.id");
        queryWrapper.orderByDesc("value");
        List<RankListVo> rankUserList = qywxDepartmentUserLinkMapper.getUserRankList(queryWrapper);
        if (rankUserList != null && rankUserList.size()>0){
            for (RankListVo rankListVo : rankUserList) {
                //查询是否有上一月记录，如果有就减去上一月传播值，没有则直接添加
                QueryWrapper<Rank> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("linkId",rankListVo.getLinkId());
                queryWrapper2.eq("type",2);//2.员工
                queryWrapper2.orderByDesc("dateStart");
                queryWrapper2.last("limit 1");
                Rank rank = rankMapper.selectOne(queryWrapper2);
                if (rank != null){
                    rankListVo.setValue(rankListVo.getValue() - rank.getValue());
                }
                Rank rank2 = new Rank();
                PropertyUtils.copyProperties(rankListVo,rank2);
                rank2.setType(2);//2.员工
                rank2.setDateStart(timeStart);
                rank2.setDateEnd(timeEnd);
                rank2.setCreateDate(new Date());
                rankMapper.insert(rank2);
            }
        }
        //计算经销商排名进排名数据库
        QueryWrapper<RankListDTO> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.groupBy("qd.departmentId");
        queryWrapper3.orderByDesc("value");
        List<RankListVo> rankDepartmentList = qywxDepartmentUserLinkMapper.getDepartmentRankList(queryWrapper3);
        if (rankDepartmentList != null && rankDepartmentList.size()>0){
            for (RankListVo rankListVo : rankDepartmentList) {
                //查询是否有上一月记录，如果有就减去上一月传播值，没有则直接添加
                QueryWrapper<Rank> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("linkId",rankListVo.getLinkId());
                queryWrapper2.eq("type",1);//1.经销商
                queryWrapper2.orderByDesc("dateStart");
                queryWrapper2.last("limit 1");
                Rank rank = rankMapper.selectOne(queryWrapper2);
                if (rank != null){
                    rankListVo.setValue(rankListVo.getValue() - rank.getValue());
                }
                Rank rank2 = new Rank();
                PropertyUtils.copyProperties(rankListVo,rank2);
                rank2.setType(1);//1.经销商
                rank2.setDateStart(timeStart);
                rank2.setDateEnd(timeEnd);
                rank2.setCreateDate(new Date());
                rankMapper.insert(rank2);
            }
        }
    }
}
