package com.zh.project0512.controller.manage;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.ReferenceGroup;
import com.zh.project0512.model.UserAccount;
import com.zh.project0512.service.IUserAccountService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-06-08
 */
@Tag(name = "用户账号关联")
@RestController
@RequestMapping("/manage/userAccount")
public class UserAccountController {

    @Autowired
    private IUserAccountService userAccountService;

    @Autowired
    private IUserService userService;

    @Data
    static class ListUAParam {
        private int pageNum;
        private int pageSize;
        @Schema(title = "userId")
        private Integer userId;
        @Schema(title = "账号状态")
        private Integer status;
    }

    @Operation(summary = "关联账号列表")
    @PostMapping("/list")
    public Result list(@Validated @RequestBody ListUAParam param) {
        QueryWrapper<UserAccount> queryWrapper = new QueryWrapper<>();
        if(param.getStatus() != null){
            queryWrapper.eq("t1.status", param.getStatus());
        }
        if(param.getUserId() != null){
            queryWrapper.eq("t1.userId", param.getUserId());
        }
        queryWrapper.orderByDesc("updateAt","creatAt");
        return Result.success(userAccountService.listAll(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()),queryWrapper));
    }

    @Data
    static class PassUAParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "用户账号id")
        private Integer id;
        @NotNull(message = "状态不能为空")
        @Min(value = 1, message = "最小值为1")
        @Max(value = 3, message = "最小值为1")
        @Schema(title = "状态",description = "1审核通过；2审核拒绝；3停用")
        private Integer status;
    }

    @Operation(summary = "关联账号审核")
    @PostMapping("/valid")
    public Result valid(@Validated @RequestBody PassUAParam param) {
        UserAccount ua = userAccountService.getById(param.getId());
        if(ua ==null){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"未找到指定账号");
        }
        if(ua.getStatus() != 0){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"账号状态不在审核中");
        }
        UpdateWrapper<UserAccount> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", param.getId()).set("status",param.getStatus()).set("updateAt",LocalDateTime.now());
        userAccountService.update(updateWrapper);
        return Result.success("修改完成");
    }
}
