package com.zh.project0512.controller.wxApp;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.Reference;
import com.zh.project0512.model.Task;
import com.zh.project0512.model.UserAccount;
import com.zh.project0512.model.validatedDemo.TaskValidGroup1;
import com.zh.project0512.service.IUserAccountService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-06-08
 */
@Tag(name = "用户账号关联")
@RestController
@RequestMapping("/wxApp/userAccount")
public class UserAccountUController {

    @Autowired
    private IUserAccountService userAccountService;

    @Autowired
    private IUserService userService;

    @Operation(summary = "添加关联账号")
    @PostMapping("/add")
    @tokenValid
    public Result add(@Validated @RequestBody UserAccount ua, @RequestHeader String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        ua.setUserId(userId).setStatus(0).setCreatAt(LocalDateTime.now());
        userAccountService.save(ua);
        return Result.success("添加完成，请等待审核");
    }

    @Operation(summary = "关联账号列表")
    @PostMapping("/list")
    @tokenValid
    public Result list(@RequestHeader String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        QueryWrapper<UserAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.orderByDesc("updateAt","creatAt");
        return Result.success(userAccountService.list(queryWrapper));
    }
}
