package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.UserReference;
import com.zh.project0512.service.IUserReferenceService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@Tag(name = "用户素材")
@RestController
@RequestMapping("/wxApp/userReference")
public class UserReferenceUController {
    @Autowired
    private IUserReferenceService userReferenceService;
    @Autowired
    private IUserService userService;

    @Data
    static class AddURParam {
        @Schema(title = "素材id")
        @NotNull(message = "referenceId不能为空")
        @Min(value = 1, message = "referenceId最小值为1")
        private Integer referenceId;
    }

    @Operation(summary = "添加到素材收藏夹")
    @tokenValid
    @PostMapping("/add")
    public Result add(@Validated @RequestBody AddURParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "用户不存在");
        }
        userReferenceService.save(new UserReference().setUserId(userId).setReferenceId(param.getReferenceId()));
        return Result.success("添加完成");
    }

    @Data
    static class AddUGRParam {
        @Schema(title = "素材id列表")
        @NotEmpty(message = "referenceId数组不能为空")
        private List<Integer> referenceIdList;
    }

    @Operation(summary = "批量添加到素材收藏夹")
    @tokenValid
    @PostMapping("/addGroup")
    public Result addGroup(@Validated @RequestBody AddUGRParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "用户不存在");
        }
        userReferenceService.addGroup(userId, param.getReferenceIdList(), LocalDateTime.now());
        return Result.success("添加完成");
    }

    @Operation(summary = "从素材收藏夹删除")
    @tokenValid
    @PostMapping("/del")
    public Result del(@Validated @RequestBody AddURParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "用户不存在");
        }
        return MybatisPlusUtil.sqlResult(userReferenceService.remove(new QueryWrapper<UserReference>().eq("userId", userId).eq("referenceId", param.getReferenceId())), "删除");
    }

    @Operation(summary = "批量从素材收藏夹删除")
    @tokenValid
    @PostMapping("/delGroup")
    public Result delGroup(@Validated @RequestBody AddUGRParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Integer userId = userService.selByOpenid(new JwtUtil().parseOpenid(token)).getId();
        if (userId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "用户不存在");
        }
        return MybatisPlusUtil.sqlResult(userReferenceService.remove(new QueryWrapper<UserReference>().eq("userId", userId).in("referenceId", param.getReferenceIdList())), "批量删除");
    }

    @Operation(summary = "用户素材列表")
    @tokenValid
    @PostMapping("/list")
    public Result list(@RequestBody(required = false) JSONObject obj, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        String openid = new JwtUtil().parseOpenid(token);
        IPage iPage = userReferenceService.dateList(MybatisPlusUtil.SetPage(obj), openid);
        List list = iPage.getRecords();
        if (list.size() > 0) {
            iPage.setRecords(userReferenceService.list(new JwtUtil().parseOpenid(token), list));
        }
        return Result.success(iPage);
    }
}
