package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.User;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.CoderUtil;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.WeChatUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@Tag(name = "用户")
@RestController
@RequestMapping("/wxApp/user")
public class UserUController {

    @Autowired
    private IUserService userService;
    @Autowired
    private WeChatUtil weChatUtil;

    @Operation(summary = "小程序登录", parameters = {@Parameter(name = "code", description = "login返回的code（未使用过！）")})
    @PostMapping("/login")
    public Result login(@RequestBody @Parameter(hidden = true) JSONObject obj) {
        JSONObject res = weChatUtil.auth(obj.getString("code"));
        String openid = res.getString("openid");
        User u = userService.selByOpenid(openid);
        if (null == u) {
            return Result.fail(HttpStatusEnum.USER_NOT_EXIST, "未注册！");
        }
        String token = new JwtUtil().createJWT(openid);
        userService.updByOpenid(openid, token, LocalDateTime.now());
        Map map = new HashMap();
        map.put("token", token);
        map.put("user", u);
        return Result.success(map);
    }

    @Operation(summary = "小程序注册", parameters = {
        @Parameter(name = "code", description = "login返回的code（未使用过！）"),
        @Parameter(name = "userInfo", description = "getUserInfo接口的iv和encryptedData"),
        @Parameter(name = "phone", description = "getPhoneNumber接口的iv和encryptedData")
    })
    @PostMapping("register")
    public Result register(@RequestBody @Parameter(hidden = true) JSONObject obj) {
        JSONObject res = weChatUtil.auth(obj.getString("code"));
        String openid = res.getString("openid");
        if (null == openid) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "无效的openid");
        }
        if (null != userService.selByOpenid(openid)) {
            return Result.fail(HttpStatusEnum.TEL_REGISTERED, "已注册");
        }
        String session_key = (String) res.get("session_key");
        JSONObject userinfo = obj.getJSONObject("userinfo");
        JSONObject phone = obj.getJSONObject("phone");
        String info = CoderUtil.decrypt(session_key, userinfo.getString("iv"), userinfo.getString("encryptedData"));
        String info2 = CoderUtil.decrypt(session_key, phone.getString("iv"), phone.getString("encryptedData"));
        JSONObject j = JSONObject.parseObject(info);
        JSONObject k = JSONObject.parseObject(info2);
        if (null == j || null == k) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "session_key解析失败");
        }
        User u = new User();
        u.setAvatar((String) j.get("avatarUrl"));
        u.setName((String) j.get("nickName"));
        u.setGender((Integer) j.get("gender"));
        u.setOpenid(openid);
        u.setTel(k.getString("phoneNumber"));
        u.setToken(new JwtUtil().createJWT(openid));
//        u.setToken(new JwtUtils().generateToken(openid, new Date().getTime()));
        u.setCreatAt(LocalDateTime.now());
        userService.save(u);
        return Result.success(u);
    }
    @PostMapping("/logout")
    @tokenValid
    public Result logout(@RequestHeader("token") @Parameter(name = "登录token") String token) {
        UpdateWrapper<User> uw = new UpdateWrapper<>();
        uw.eq("token",token).set("token",null).set("updateAt",LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(userService.update(uw),"登出");
    }
}
