package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.annotation.tokenValid;
import com.zh.project0512.model.Video;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.service.IVideoService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@RestController
@RequestMapping("/wxApp/video")
@Tag(name = "视频")
public class VideoUController {
    @Autowired
    private IVideoService videoService;
    @Autowired
    private IUserService userService;

    @Data
    static class UserAddParam {
        @NotNull(message = "任务id不能为空")
        private Integer id;
        @NotNull(message = "type不能为空")
        @Min(value = 1, message = "视频范围为[1.抖音,2.快手]")
        @Max(value = 2, message = "视频范围为[1.抖音,2.快手]")
        @Schema(title = "平台", description = "1抖音；2快手")
        private Integer type;
        @NotNull(message = "视频地址不能为空")
        private String url;
    }

    @Operation(summary = "用户上传")
    @PostMapping("userAdd")
    @tokenValid
    public Result userAdd(@Validated @RequestBody UserAddParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Claims claims = new JwtUtil().parseJWT(token);
        Integer useId = userService.selByOpenid(claims.getId()).getId();
        if (useId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        videoService.save(new Video().setUserId(useId).setTaskId(param.getId()).setType(param.getType()).setUrl(param.getUrl()).setCreatAt(LocalDateTime.now()));
        return Result.success("添加完成！");
    }

    @Operation(summary = "用户上传记录")
    @PostMapping("addedRecord")
    @tokenValid
    public Result addedRecord(@RequestBody(required = false) JSONObject obj, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Claims claims = new JwtUtil().parseJWT(token);
        Integer useId = userService.selByOpenid(claims.getId()).getId();
        if (useId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        QueryWrapper<Video> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId",useId);
        Integer taskId =  obj.getInteger("taskId");
        if(taskId != null){
            queryWrapper.eq("taskId",taskId);
        }
        return Result.success(videoService.page(MybatisPlusUtil.SetPage(obj),queryWrapper));
    }
}
