package com.zh.project0512.utils;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.net.ssl.HttpsURLConnection;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;

/**
 * http工具类
 */
@Slf4j
@Component
public class HttpUtil {
    @Value("${qywx.corpid}")
    private String corpid;
    @Value("${qywx.corpsecret}")
    private String corpsecret;

    public String qywxGetToken(){
        String url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";
        String requestUrl = url + "?corpid=" + corpid + "&corpsecret=" + corpsecret;
        String str = httpRequest(requestUrl, "GET", null);
        return JSONObject.parseObject(str).getString("access_token");
    }
    public  JSONObject qywxDepartmentList(String access_token){
        String url = "https://qyapi.weixin.qq.com/cgi-bin/department/list";
        String requestUrl = url + "?access_token=" + access_token;
        String str = httpRequest(requestUrl, "GET", null);
        return JSONObject.parseObject(str);
    }
    public  JSONObject qywxUserSession(String access_token,String code){
        String url = "https://qyapi.weixin.qq.com/cgi-bin/miniprogram/jscode2session";
        String requestUrl = url + "?access_token=" + access_token +"&js_code="+ code +"&grant_type=authorization_code";
        String str = httpRequest(requestUrl, "GET", null);
        return JSONObject.parseObject(str);
    }
    public  JSONObject qywxUser(String access_token,String userid){
        String url = "https://qyapi.weixin.qq.com/cgi-bin/user/get";
        String requestUrl = url + "?access_token=" + access_token +"&userid="+ userid ;
        String str = httpRequest(requestUrl, "GET", null);
        return JSONObject.parseObject(str);
    }

    public  Integer qywxDepartmentParentid(String access_token,int id,int loop){
        String url = " https://qyapi.weixin.qq.com/cgi-bin/department/get";
        String requestUrl = url + "?access_token=" + access_token +"&id="+ id ;
        String str = httpRequest(requestUrl, "GET", null);
        Integer parentid = JSONObject.parseObject(str).getJSONObject("department").getInteger("parentid");
        if( parentid != 1 && loop< 8 ){
            qywxDepartmentParentid(access_token,parentid,++loop);
        }
        return parentid;
    }
    public static String httpRequest(String requestUrl, String requestMethod, String output) {
        try {
            URL url = new URL(requestUrl);
            HttpsURLConnection connection = (HttpsURLConnection) url.openConnection();
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setUseCaches(false);
            connection.setRequestMethod(requestMethod);
            if (null != output) {
                OutputStream outputStream = connection.getOutputStream();
                outputStream.write(output.getBytes(StandardCharsets.UTF_8));
                outputStream.close();
            }
            // 从输入流读取返回内容
            InputStream inputStream = connection.getInputStream();
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, StandardCharsets.UTF_8);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            String str;
            StringBuilder buffer = new StringBuilder();
            while ((str = bufferedReader.readLine()) != null) {
                buffer.append(str);
            }
            bufferedReader.close();
            inputStreamReader.close();
            inputStream.close();
            connection.disconnect();
            return buffer.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 向指定 URL 发送POST方法的请求
     *
     * @param url  发送请求的 URL
     * @param json 请求参数，请求参数应该是 json 的形式。
     * @return 所代表远程资源的响应结果
     */
    public static String httpPost(String url, JSONObject json) {
        PrintWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent",
                "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 获取URLConnection对象对应的输出流
            out = new PrintWriter(conn.getOutputStream());
            // 发送请求参数

            out.print(json);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(
                new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result=result.concat(line);
            }
        } catch (Exception e) {
            System.out.println("发送 POST 请求出现异常！" + e);
            e.printStackTrace();
        }
        //使用finally块来关闭输出流、输入流
        finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        return result;
    }

}
