package com.zh.project0512.utils.result;

import lombok.Data;

import java.util.List;

/**
 * @author jinliwei
 * 返回list集合
 */
@Data
public class ResultList<T> {

    public ResultList(boolean success, int code) {
        this.setSuccess(success);
        this.setCode(code);
    }

    public ResultList(boolean success, int code, List<T> data) {
        this.setSuccess(success);
        this.setCode(code);
        this.setData(data);
    }

    public ResultList(boolean success, int code, String msg) {
        this.setSuccess(success);
        this.setCode(code);
        this.setData(data);
    }

    public ResultList(boolean success, int code, List<T> data, String msg) {
        this.setSuccess(success);
        this.setCode(code);
        this.setData(data);
        this.setMsg(msg);
    }

    /**
     * 请求是否成功
     * true:成功
     * false：失败
     */
    private boolean success;

    /**
     * 状态码
     * 成功：200
     * 失败：其他
     */
    private int code;

    /**
     * 失败状态码描述
     * 如果成功不返回
     * 失败返回状态码对应的msg消息
     */
    private String msg;

    /**
     * 请求数据的结果
     */
    private List<T> data;


    public static <T> ResultList<T> success() {
        return new ResultList<T>(true, 200);
    }


    public static <T> ResultList<T> success(List<T> data) {
        return new ResultList<T>(true, 200, data);
    }

    public static <T> ResultList<T> success(List<T> data, String msg) {
        return new ResultList<T>(true, 200, data, msg);
    }

    public static <T> ResultList<T> fail(HttpStatusEnum httpStatusEnum) {
        return new ResultList<T>(false, httpStatusEnum.code(), httpStatusEnum.reasonPhraseUS());
    }

    public static <T> ResultList<T> fail(HttpStatusEnum httpStatusEnum, String msg) {
        return new ResultList<T>(false, httpStatusEnum.code(), msg);
    }


}
